@@ -6,13 +6,15 @@ use parent 'Exporter::Tiny';
 use Web::ComposableRequest::Exception;
 use Role::Tiny ();
 
-our @EXPORT = qw( EXCEPTION_CLASS FALSE LANG NUL TRUE );
+our @EXPORT = qw( COMMA EXCEPTION_CLASS FALSE LANG NUL SPC TRUE );
 
 my $Exception_Class = 'Web::ComposableRequest::Exception';
 
+sub COMMA () { q(,) }
 sub FALSE () { 0    }
 sub LANG  () { 'en' }
 sub NUL   () { q()  }
+sub SPC   () { q( ) }
 sub TRUE  () { 1    }
 
 sub EXCEPTION_CLASS () { __PACKAGE__->Exception_Class }
